package choco.bool;

import choco.AbstractConstraint;
import choco.Constraint;
import choco.AbstractVar;
import choco.mem.StoredInt;
import choco.util.BitSet;

// **************************************************
// *                   J-CHOCO                      *
// *   Copyright (C) F. Laburthe, 1999-2004         *
// **************************************************
// *  an open-source Constraint Programming Kernel  *
// *     for Research and Education                 *
// **************************************************

/* FileAbstractBinBoolConstraint.java, last modified by Fran?ois, 28 mars 2004 */

/**
 * An abstract class for binary Boolean composition
 */
public abstract class AbstractBinBoolConstraint extends AbstractBinCompositeConstraint implements BoolConstraint {

  /**
   * using a bitvector to store four three-valued data (true, false, unknown): status/targetStatus of c.const0/2
   * Each data is stored on two bits:
   * - the first bit indicates whether the data is known or unknown
   * - the second bit indicates whether the data is true or false
   * Therefore, the second bit is meaningful only when the first one is true.
   */
  protected StoredInt statusBitVector;

  public AbstractBinBoolConstraint(AbstractConstraint c1, AbstractConstraint c2) {
    super(c1, c2);
    statusBitVector = new StoredInt(getProblem().getEnvironment(), 0);
  }

  public Boolean getStatus(int constIdx) {
    if (BitSet.getBit(statusBitVector.get(), 4 * constIdx)) {
      if (BitSet.getBit(statusBitVector.get(), 4 * constIdx + 1))
        return Boolean.TRUE;
      else
        return Boolean.FALSE;
    } else
      return null;
  }

  public Boolean getTargetStatus(int constIdx) {
    if (BitSet.getBit(statusBitVector.get(), 4 * constIdx + 2)) {
      if (BitSet.getBit(statusBitVector.get(), 4 * constIdx + 3))
        return Boolean.TRUE;
      else
        return Boolean.FALSE;
    } else
      return null;
  }

  public void setStatus(int constIdx, boolean st) {
    assert((getStatus(constIdx) == null));                // TODO! move into a junit test
    statusBitVector.set(BitSet.setBit(statusBitVector.get(), 4 * constIdx));
    if (st) {
      if (getStatus(constIdx) == Boolean.FALSE)
        statusBitVector.set(BitSet.setBit(statusBitVector.get(), 4 * constIdx + 1));
    } else {
      if (getStatus(constIdx) == Boolean.TRUE)
        statusBitVector.set(BitSet.unsetBit(statusBitVector.get(), 4 * constIdx + 1));
    }
    assert(!(getStatus(constIdx) == null));                // TODO! move into a junit test
    assert((getStatus(constIdx) == Boolean.valueOf(st)));  // TODO! move into a junit test
  }

  public void setTargetStatus(int constIdx, boolean st) {
    assert(!(getTargetStatus(constIdx) == null));             // TODO! move into a junit test
    statusBitVector.set(BitSet.setBit(statusBitVector.get(), 4 * constIdx + 2));
    if (st) {
      if (getTargetStatus(constIdx) == Boolean.FALSE)
        statusBitVector.set(BitSet.setBit(statusBitVector.get(), 4 * constIdx + 3));
    } else {
      if (getTargetStatus(constIdx) == Boolean.TRUE)
        statusBitVector.set(BitSet.unsetBit(statusBitVector.get(), 4 * constIdx + 3));
    }
    assert(!(getTargetStatus(constIdx) == null));                // TODO! move into a junit test
    assert((getTargetStatus(constIdx) == Boolean.valueOf(st)));  // TODO! move into a junit test
  }

  public void setSubConstraintStatus(Constraint subConstraint, boolean status, int varOffset) {
    int subConstIdx = getSubConstraintIdx(varOffset);
    if (subConstraint == getSubConstraint(subConstIdx)) {
       setStatus(subConstIdx, status);
    } else {
      BoolConstraint newRoot = (BoolConstraint) getSubConstraint(subConstIdx);
      newRoot.setSubConstraintStatus(subConstraint, status, this.offset);
    }
  }

}
