// **************************************************
// *                   J-CHOCO                      *
// *   Copyright (C) F. Laburthe, 1999-2003         *
// **************************************************
// *  an open-source Constraint Programming Kernel  *
// *     for Research and Education                 *
// **************************************************

/* File choco.bool.AbstractBinCompositeConstraint.java, last modified by FLABURTHE 17 mars 2004 */
package choco.bool;

import choco.AbstractConstraint;
import choco.Var;
import choco.Constraint;
import choco.AbstractVar;
import choco.integer.IntVar;

public abstract class AbstractBinCompositeConstraint extends AbstractCompositeConstraint {
  /**
   * the first sub-constraint of the composition
   */
  protected AbstractConstraint const0;
  /**
   * the second sub-constraint of the composition
   */
  protected AbstractConstraint const1;

  /**
   * <i>Network management:</i>
   * Accessing the i-th search variable of a constraint.
   *
   * @param varIdx index of the variable among all search variables in the constraint. Numbering start from 0 on.
   * @return the variable, or null when no such variable is found
   */

  public IntVar getIntVar(int varIdx) {
    return (IntVar) getVar(varIdx);
  }

  /**
   * the number of variables in the first sub-constraint
   * (therefore the offset in the numbering of the variables from the second sub-constraint)
   */
  protected int offset = 0;

  public AbstractBinCompositeConstraint(AbstractConstraint c1, AbstractConstraint c2) {
    const0 = c1;
    const1 = c2;
    offset = const0.getNbVars();
  }

  public int assignIndices(AbstractCompositeConstraint root, int i) {
    int j = i;
    j = const0.assignIndices(root, j);
    this.offset = j - i;
    j = const1.assignIndices(root, j);
    return j;
  }
  
  public int getSubConstraintIdx(int varIdx) {
    if (varIdx < offset) {
      return 0;
    } else {
      return 1;
    }
  }

  public int getVarIdxInOpposite(int i) {
    int constIdx = getSubConstraintIdx(i);
    if (constIdx == 0) {
      return const0.getVarIdxInOpposite(i);
    } else {
      assert(constIdx == 1);
      return const1.getVarIdxInOpposite(i - offset) + offset;
    }
  }

  public int getNbVars() {
    return const0.getNbVars() + const1.getNbVars();
  }

  public Var getVar(int i) {
    return ((i < offset) ? const0.getVar(i) : const1.getVar(i - offset));
  }

  public boolean isCompletelyInstantiated() {
    return (const0.isCompletelyInstantiated() && const1.isCompletelyInstantiated());
  }

  public int getConstraintIdx(int i) {
    return ((i < offset) ? const0.getConstraintIdx(i) : const1.getConstraintIdx(i - offset));
  }

  public void setConstraintIndex(int i, int idx) {
    if (i < offset) {
      const0.setConstraintIndex(i, idx);
    } else {
      const1.setConstraintIndex(i - offset, idx);
    }
  }

  public AbstractConstraint getSubConstraint(int constIdx) {
    return ((constIdx == 0) ? const0 : const1);
  }

  public int getNbSubConstraints() {
    return 2;
  }
}
