// *********************************************
// *                   J-CHOCO                 *
// *   Copyright (c) F. Laburthe, 1999-2003    *
// *********************************************
// * Event-base contraint programming Engine   *
// *********************************************

// CVS Information
// File:               $RCSfile: AbstractCompositeConstraint.java,v $
// Version:            $Revision: 1.1.1.1 $
// Last Modification:  $Date: 2004/05/11 13:51:45 $
// Last Contributor:   $Author: laburthe $

package choco.bool;

import choco.*;
import choco.util.IntIterator;

/**
 * An abstract class for all implementations of listeners over search variables.
 */
public abstract class AbstractCompositeConstraint extends AbstractConstraint implements CompositeConstraint {
  Constraint[] subConstraints;
  int[] offsets;
  Var[] additionalVars;
  int[] additionalIndices;

  public void awakeOnBounds(int varIndex) throws ContradictionException {
    awakeOnInf(varIndex);
    awakeOnSup(varIndex);
  }

  public void awakeOnRemovals(int idx, IntIterator deltaDomain) throws ContradictionException {
    if (deltaDomain != null) {
      for (; deltaDomain.hasNext();) {
        int val = deltaDomain.next();
        awakeOnRem(idx, val);
      }
    } else {
      awakeOnVar(idx);
    }
  }

  public int getGlobalVarIndex(Constraint subConstraint, int localVarIdx) {
    AbstractVar v = (AbstractVar) subConstraint.getVar(localVarIdx);
    int constIdx = subConstraint.getConstraintIdx(localVarIdx);
    return v.getVarIndex(constIdx);
  }

  public void addListener() {
    assignIndices(this,-1);
    if (this.hook != null) this.hook.addListener();
  }

}