package choco.bool;

import choco.Constraint;
import choco.mem.Environment;
import choco.mem.StoredInt;
import choco.mem.StoredIntVector;
import choco.util.BitSet;
import choco.Constraint;

// **************************************************
// *                   J-CHOCO                      *
// *   Copyright (C) F. Laburthe, 1999-2004         *
// **************************************************
// *  an open-source Constraint Programming Kernel  *
// *     for Research and Education                 *
// **************************************************

/* FileAbstractLargeBoolConstraint.java, last modified by Fran?ois, 28 mars 2004 */

/**
 * An abstract class for Boolean composition of more than two constraints
 */
public abstract class AbstractLargeBoolConstraint extends AbstractLargeCompositeConstraint implements BoolConstraint {
  protected StoredIntVector statusBitVector;
  protected StoredInt nbTrueStatus;
  protected StoredInt nbFalseStatus;

  public AbstractLargeBoolConstraint(Constraint[] subConstraints) {
    super(subConstraints);
    Environment env = getProblem().getEnvironment();
    statusBitVector = env.makeIntVector((nbConstraints - 1) / 8 + 1, 0);
    nbTrueStatus = (StoredInt) env.makeInt(0);
    nbFalseStatus = (StoredInt) env.makeInt(0);
  }

  public Boolean getStatus(int constIdx) {
    int i1 = (constIdx / 8);
    int i2 = 4 * (constIdx % 8);
    if (BitSet.getBit(statusBitVector.get(i1), i2)) {
      if (BitSet.getBit(statusBitVector.get(i1), i2 + 1)) {
        return Boolean.TRUE;
      } else {
        return Boolean.FALSE;
      }
    } else {
      return null;
    }
  }

  public Boolean getTargetStatus(int constIdx) {
    int i1 = (constIdx / 8);
    int i2 = 4 * (constIdx % 8);
    if (BitSet.getBit(statusBitVector.get(i1), i2 + 2)) {
      if (BitSet.getBit(statusBitVector.get(i1), i2 + 3)) {
        return Boolean.TRUE;
      } else {
        return Boolean.FALSE;
      }
    } else {
      return null;
    }
  }

  public void setStatus(int constIdx, boolean st) {
    assert(getStatus(constIdx) == null);
    int i1 = (constIdx / 8);
    int i2 = 4 * (constIdx % 8);
    statusBitVector.set(i1, BitSet.setBit(statusBitVector.get(i1), i2));
    if (st) {
      if (getStatus(constIdx) == Boolean.FALSE)
        statusBitVector.set(i1, BitSet.setBit(statusBitVector.get(i1), i2 + 1));
    } else {
      if (getStatus(constIdx) == Boolean.TRUE)
        statusBitVector.set(i1, BitSet.unsetBit(statusBitVector.get(i1), i2 + 1));
    }
    assert(getStatus(constIdx) == Boolean.valueOf(st));
  }

  public void setTargetStatus(int constIdx, boolean st) {
    assert(getTargetStatus(constIdx) == null);
    int i1 = (constIdx / 8);
    int i2 = 4 * (constIdx % 8);
    statusBitVector.set(i1, BitSet.setBit(statusBitVector.get(i1), i2 + 2));
    if (st) {
      if (getTargetStatus(constIdx) == Boolean.FALSE)
        statusBitVector.set(i1, BitSet.setBit(statusBitVector.get(i1), i2 + 3));
    } else {
      if (getTargetStatus(constIdx) == Boolean.TRUE)
        statusBitVector.set(i1, BitSet.unsetBit(statusBitVector.get(i1), i2 + 3));
    }
    assert(getTargetStatus(constIdx) == Boolean.valueOf(st));
  }


  public int assignIndices(AbstractCompositeConstraint root, int i) {
    int j = i;
    for (int constIdx = 0; constIdx < nbConstraints; constIdx++) {
      j = constraints[constIdx].assignIndices(root, i);
    }
    for (int k = 0; k < additionalVars.length; k++) {
      j++;
      root.connectVar(additionalVars[k], j);
    }
    return j;
  }

  public void setSubConstraintStatus(Constraint subConstraint, boolean status, int varOffset) {
    int subConstIdx = getSubConstraintIdx(varOffset);
    if (subConstraint == getSubConstraint(subConstIdx)) {
       setStatus(subConstIdx, status);
    } else {
      BoolConstraint newRoot = (BoolConstraint) getSubConstraint(subConstIdx);
      int deltaOffset = (subConstIdx == 0) ? 0 : offsets[subConstIdx];
      newRoot.setSubConstraintStatus(subConstraint, status, varOffset - deltaOffset);
    }
  }
  
}