package choco.bool;

import choco.AbstractConstraint;
import choco.Constraint;

// **************************************************
// *                   J-CHOCO                      *
// *   Copyright (C) F. Laburthe, 1999-2003         *
// **************************************************
// *  an open-source Constraint Programming Kernel  *
// *     for Research and Education                 *
// **************************************************

/* File choco.bool.AbstractLargeBoolConstraintWithCounterOpposite.java, last modified by flaburthe 30 mars 2004 15:34:29 */

/**
 * An abstract class storing the counterpart of each subconstraint + an index correspondence for variables
 * (between subconstraints and opposite subconstraints)
 */
public abstract class AbstractLargeBoolConstraintWithCounterOpposite extends AbstractLargeBoolConstraint {

  /**
   * the counterpart of the first subconstraint
   */
  AbstractConstraint[] opposites;

  /**
   * the index correspondance for the variables of the subconstraints and their counterparts
   */
  int[][] indicesInOpposites;


  public AbstractLargeBoolConstraintWithCounterOpposite(Constraint[] subConstraints) {
    super(subConstraints);
    int nbc = getNbSubConstraints();
    opposites = new AbstractConstraint[nbc];
    for (int constIdx = 0; constIdx < nbc; constIdx++) {
      AbstractConstraint subConst = getSubConstraint(constIdx);
      opposites[constIdx] = subConst.opposite();
      for (int varIdx = 0; varIdx < subConst.getNbVars(); varIdx++) {
        subConst.getVarIdxInOpposite(varIdx);
      }
    }

    indicesInOpposites = new int[nbc][];
    for (int constIdx = 0; constIdx < nbc; constIdx ++) {
      AbstractConstraint subConst = getSubConstraint(constIdx);
      indicesInOpposites[constIdx] = new int[subConst.getNbVars()];
      for (int k = 0; k < subConst.getNbVars(); k++) {
        indicesInOpposites[constIdx][k] = subConst.getVarIdxInOpposite(k);
      }
    }
  }

  public int assignIndices(AbstractCompositeConstraint root, int i) {
    int j = i;
    for (int constIdx = 0; constIdx < this.nbConstraints; constIdx++) {
      int j0 = j;
      AbstractConstraint subc = constraints[constIdx];
      j = subc.assignIndices(root, j);
      assert(offsets[constIdx] == j - i);
      for (int varIdx = 0; varIdx < j - j0; varIdx++) {
        opposites[constIdx].setConstraintIndex(indicesInOpposites[constIdx][varIdx], subc.getConstraintIdx(varIdx));
      }
    }
    assert(additionalIndices.length == additionalVars.length);
    for (int varIdx = 0; varIdx < additionalVars.length; varIdx++) {
      j++;
      root.connectVar(additionalVars[varIdx], j);
      this.additionalIndices[varIdx] = additionalVars[varIdx].getNbConstraints();
    }
    return j;
  }

  public Constraint getOppositeSubConstraint(int subConstIdx) {
    return opposites[subConstIdx];
  }

}