package choco.bool;

import choco.AbstractConstraint;
import choco.ContradictionException;
import choco.Constraint;
import choco.integer.IntConstraint;
import choco.integer.constraints.EqualXYC;
import choco.util.IntIterator;

// **************************************************
// *                   J-CHOCO                      *
// *   Copyright (C) F. Laburthe, 1999-2004         *
// **************************************************
// *  an open-source Constraint Programming Kernel  *
// *     for Research and Education                 *
// **************************************************

/* FileBinConjunction.java, last modified by Franois, 4 avr. 2004 */

/**
 * A class for Conjunctions (only used in subterms of Boolean formulae (c1 and c2)
 */
// note v1.02: for conjunctions, targetStatus slots are useless -> we only use status fields
public class BinConjunction extends AbstractBinBoolConstraintWithCounterOpposite {

  public BinConjunction(AbstractConstraint c0, AbstractConstraint c1) {
    super(c0, c1);
  }

  public void awakeOnInf(int idx) throws ContradictionException {
    if (idx < offset) {
      ((IntConstraint) const0).awakeOnInf(idx);
    }
  }

  public void awakeOnSup(int idx) throws ContradictionException {
    if (idx < offset) {
      ((IntConstraint) const0).awakeOnSup(idx);
    }
  }

  public void awakeOnInst(int idx) throws ContradictionException {
    if (idx < offset) {
      ((IntConstraint) const0).awakeOnInst(idx);
    }
  }

  public void awakeOnRem(int idx, int x) throws ContradictionException {
    if (idx < offset) {
      ((IntConstraint) const0).awakeOnRem(idx, x);
    }
  }

  public void awakeOnRemovals(int idx, IntIterator deltaDomain) throws ContradictionException {
    if (idx < offset) {
      ((IntConstraint) const0).awakeOnRemovals(idx, deltaDomain);
    }
  }

  public void awakeOnBounds(int varIndex) throws ContradictionException {
    if (varIndex < offset) {
      ((IntConstraint) const0).awakeOnBounds(varIndex);
    }
  }

  public void propagate() throws ContradictionException {
    const0.propagate();
    const1.propagate();
  }

  public void awake() throws ContradictionException {
    const0.awake();
    const1.awake();
  }

  public boolean isSatisfied() {
    return (const0.isSatisfied() && const1.isSatisfied());
  }

  public boolean isConsistent() {
    throw new UnsupportedOperationException();
  }

  public Boolean isEntailed() {
    Boolean B0 = getStatus(0);
    if (B0 == null) {
      B0 = const0.isEntailed();
      if (B0 != null) {
        setStatus(0, B0.booleanValue());
      }
    }
    Boolean B1 = getStatus(1);
    if (B1 == null) {
      B1 = const1.isEntailed();
      if (B1 != null) {
        setStatus(1, B1.booleanValue());
      }
    }
    if ((B0 == Boolean.TRUE) && (B1 == Boolean.TRUE)) {
      return Boolean.TRUE;
    } else if ((B0 == Boolean.FALSE) || (B1 == Boolean.FALSE)) {
      return Boolean.FALSE;
    } else {
      return null;
    }
  }

  public AbstractConstraint opposite() {
    return new BinDisjunction(const0.opposite(), const1.opposite());
  }

  public boolean isEquivalentTo(Constraint compareTo) {
    if (compareTo instanceof BinConjunction) {
      BinConjunction c = (BinConjunction) compareTo;
      return ( ( this.const0.isEquivalentTo(c.const0) && this.const1.isEquivalentTo(c.const1) )  ||
               ( this.const0.isEquivalentTo(c.const1) && this.const1.isEquivalentTo(c.const0) )  );
    } else {
      return false;
    }
  }

}
