// *********************************************
// *                   J-CHOCO                 *
// *   Copyright (c) F. Laburthe, 1999-2003    *
// *********************************************
// * Event-base contraint programming Engine   *
// *********************************************

// CVS Information
// File:               $RCSfile: AbstractUnIntConstraint.java,v $
// Version:            $Revision: 1.2 $
// Last Modification:  $Date: 2004/06/30 11:12:26 $
// Last Contributor:   $Author: grochart $

package choco.integer.constraints;

import choco.ContradictionException;
import choco.Var;
import choco.AbstractVar;
import choco.bool.AbstractCompositeConstraint;

//import choco.choco.*;
//import choco.search.AbstractIntConstraint;

/**
 * An abstract class for all implementations of (unary) listeners over
 * one search variable.
 */
public abstract class AbstractUnIntConstraint extends AbstractIntConstraint {

  /**
   * The unique variable of the constraint.
   */

  public choco.integer.IntVar v0;


  /**
   * The index of the constraint among all listeners of its first (and unique)
   * variable.
   */

  public int cIdx0;


  /**
   * Let v be the i-th var of c, records that c is the n-th constraint involving v.
   */

  public void setConstraintIndex(int i, int val) {
    if (i == 0)
      cIdx0 = val;
    else
      throw new Error("bug in setConstraintIndex i:" + i + " this: " + this);
  }


  /**
   * Returns the index of this listeners in the variable <code>idx</code>.
   * @param idx Index of the variable.
   */

  public int getConstraintIdx(int idx) {
    if (idx == 0)
      return cIdx0;
    else
      return -1;
  }


  /**
   * Checks if all the variables of the constraint are instantiated.
   */

  public boolean isCompletelyInstantiated() {
    return v0.isInstantiated();
  }


  /**
   * Returns the number of variables: 1 for an unIntConstraint.
   */

  public int getNbVars() {
    return (1);
  }


  /**
   * Returns the variable number <code>i</code>. Here, <code>i</code>
   * should be 0.
   */

  public Var getVar(int i)  {
    if (i == 0)
      return v0;
    else
      return null;
  }


  /**
   * Propagation on the constraint awake var.
   * @throws choco.ContradictionException
   */

  public void propagate() throws ContradictionException {
    // Ca risque de boucler avec cette definition car awakeOnVar est defini
    // par propagate...
    awakeOnVar(1);
  }


  /**
   * Adds this constraint as new constraint in each variable.
   */

  public void addConstraint() {
    setConstraintIndex(0, v0.addConstraint(this, 0, true));
    if (this.hook != null) this.hook.addListener();
  }


  /**
   * Deactivates this listener in all the variables.
   */

  public void deactivateConstraint() {
    v0.deactivateConstraint(cIdx0);
    if (this.hook != null) this.hook.deactivateListener();
  }


  /**
   * Activates this listener in all the variables.
   */

  public void activateConstraint() {
    v0.activateConstraint(cIdx0);
    if (this.hook != null) this.hook.activateListener();
  }


  /**
   * Checks if the constraint is active.
   */

  public boolean isActive() {
    return v0.isActive(cIdx0);
  }

  /**
   * Gets the <code>i</code>th search valued variable.
   */

  public choco.integer.IntVar getIntVar(int i) {
    if (i == 0)
      return v0;
    else
      return null;
  }

  public int assignIndices(AbstractCompositeConstraint root, int i) {
    int j=i;
    j++; root.connectVar((AbstractVar) v0, j);
    setConstraintIndex(0, v0.getNbConstraints() - 1);
    return j;
  }

}
