package choco.integer.var;

import choco.ContradictionException;
import choco.mem.StoredInt;
import choco.mem.Environment;
import choco.util.IntIterator;

// *********************************************
// *                   J-CHOCO                 *
// *   Copyright (c) F. Laburthe, 1999-2003    *
// *********************************************
// * Event-base contraint programming Engine   *
// *********************************************

// CVS Information
// File:               $RCSfile: IntervalIntDomain.java,v $
// Version:            $Revision: 1.1.1.1 $
// Last Modification:  $Date: 2004/05/11 13:51:47 $
// Last Contributor:   $Author: laburthe $

public class IntervalIntDomain extends AbstractIntDomain {
  /**
   * The backtrackable minimal value of the variable.
   */

  protected final StoredInt inf;

  /**
   * The backtrackable maximal value of the variable.
   */

  protected final StoredInt sup;

  public IntervalIntDomain(choco.integer.var.IntDomainVar v, int a, int b) {
    variable = v;
    problem = v.getProblem();
    Environment env = problem.getEnvironment();
    inf = (StoredInt) env.makeInt(a);
    sup = (StoredInt) env.makeInt(b);
  }

  public boolean contains(int x) {
    return ((x >= getInf()) && (x <= getSup()));
  }

  public int getNextValue(int x) {
    if (x < getInf()) {
      return getInf();
    } else if (x < getSup()) {
      return x + 1;
    } else {
      return Integer.MAX_VALUE;
    }
  }

  public int getPrevValue(int x) {
    if (x > getSup()) {
      return getSup();
    } else if (x > getInf()) {
      return x - 1;
    } else {
      return Integer.MIN_VALUE;
    }
  }

  public int getRandomValue() {
    // TODO
    return 0;
  }

  public int getSize() {
    return getSup() - getInf() + 1;
  }

  public boolean hasNextValue(int x) {
    return (x < getSup());
  }

  public boolean hasPrevValue(int x) {
    return (x > getInf());
  }

  public IntIterator getIterator() {
    // TODO
    return null;
  }

  public boolean remove(int x) {
    return false;
  }

  public int getSup() {
      return sup.get();
    }

    public int getInf() {
      return inf.get();
    }

  public void restrict(int x) {
    inf.set(x);
    sup.set(x);
  }

  public int updateInf(int x) {
    inf.set(x);
    return x;
  }

  public int updateSup(int x) {
    sup.set(x);
    return x;
  }

  protected boolean _removeVal(int x) throws ContradictionException {
    int infv = getInf(), supv = getSup();
    if (x == infv) {
      _updateInf(x + 1);
      if (getInf() == supv) _instantiate(supv);
      return true;
    } else if (x == supv) {
      _updateSup(x - 1);
      if (getSup() == infv) _instantiate(infv);
      return true;
    } else {
      return false;
    }
  }

  public boolean isEnumerated() {
    return false;
  }

  public IntIterator getDeltaIterator() {
    return null;
  }

  public String pretty() {
    return ("[" + this.getInf() + ", " + this.getSup() + "]");
  }

}
