// *********************************************
// *                   J-CHOCO                 *
// *   Copyright (c) F. Laburthe, 1999-2003    *
// *********************************************
// * Event-base contraint programming Engine   *
// *********************************************

// CVS Information
// File:               $RCSfile: StoredInt.java,v $
// Version:            $Revision: 1.1.1.1 $
// Last Modification:  $Date: 2004/05/11 13:51:47 $
// Last Contributor:   $Author: laburthe $


package choco.mem;

import java.util.logging.Logger;

/**
 * A class implementing backtrackable integers.
 */
public final class StoredInt implements IStateInt {
  /**
   * Reference to an object for logging trace statements related memory & backtrack (using the java.util.logging package)
   */

  private static Logger logger = Logger.getLogger("choco.mem");

  /**
   * The current {@link choco.mem.Environment}.
   */

  private Environment environment;


  /**
   * Current value of the search.
   */

  private int currentValue;


  /**
   * The last world the search was moidified in.
   */

  int worldStamp;


  /**
   *  The current {@link choco.mem.StoredIntTrail}.
   */

  private final StoredIntTrail trail;


  /**
   * Constructs a stored search with an unknown initial value.
   * Note: this constructor should not be used directly: one should instead
   * use the Environment factory
   */

  public StoredInt(Environment env) {
    this(env, UNKNOWN_INT);
  }


  /**
   * Constructs a stored search with an initial value.
   * Note: this constructor should not be used directly: one should instead
   * use the Environment factory
   */

  public StoredInt(Environment env, int i) {
    environment = env;
    currentValue = i;
    worldStamp = env.currentWorld;
    trail = (StoredIntTrail) this.environment.getTrail(Environment.INT_TRAIL);
  }


  /**
   * Returns the current value.
   */

  public int get() {
    return currentValue;
  }


  /**
   * Checks if a value is currently stored.
   */

  public boolean isKnown() {
    return (currentValue != UNKNOWN_INT);
  }


  /**
   * Modifies the value and stores if needed the former value on the
   * trailing stack.
   */

  public void set(int y) {
    if (y != currentValue) {
      if (this.worldStamp < environment.currentWorld) {
        trail.savePreviousState(this, currentValue, worldStamp);
        worldStamp = environment.currentWorld;
      }
      currentValue = y;
    }
  }

  /**
   * modifying a StoredInt by an increment
   * @param delta
   */
  public void add(int delta) {
    set(get() + delta);
  }

  /** Modifies the value without storing the former value on the trailing stack.
   * @param y the new value
   * @param wstamp the stamp of the world in which the update is performed */

  void _set(int y, int wstamp) {
    currentValue = y;
    worldStamp = wstamp;
  }

  /**
   * Retrieving the environment
   */
  public Environment getEnvironment() {
    return environment;
  }

}

