// *********************************************
// *                   J-CHOCO                 *
// *   Copyright (c) F. Laburthe, 1999-2003    *
// *********************************************
// * Event-base contraint programming Engine   *
// *********************************************

// CVS Information
// File:               $RCSfile: StoredVectorTrail.java,v $
// Version:            $Revision: 1.1.1.1 $
// Last Modification:  $Date: 2004/05/11 13:51:47 $
// Last Contributor:   $Author: laburthe $

package choco.mem;


/**
 * Implements storage of historical values for backtrackable integers
 */
public class StoredVectorTrail implements ITrailStorage {
  /** reference towards the overall environment (responsible for all memory management) */
  public Environment environment;

  /** stack of backtrackable vectors */
  private StoredVector[] vectorStack;

  /** stack of indices */
  private int[] indexStack;

  /** stack of values (former values that need be restored upon backtracking */
  private Object[] valueStack;

  /** a pointer indicating the level of the last entry */
  private int currentLevel;

  /** a stack of pointers (for each start of a world) */
  private int[] worldStartLevels;

  /**
   * capacity of the trailing stack (in terms of number of updates that can be stored)
   */
  private int maxUpdates = 0;

  /**
   * capacity of the trailing stack (in terms of number of worlds that can be handled)
  */
  private int maxWorlds = 0;

  /** constructor: building a trail with predefined size
   * @param nUpdates maximal number of updates that will be stored
   * @param nWorlds  maximal number of worlds that will be stored
   */
  public StoredVectorTrail(Environment env, int nUpdates, int nWorlds) {
    environment = env;
    currentLevel = 0;
    vectorStack = new StoredVector[nUpdates];
    indexStack = new int[nUpdates];
    valueStack = new Object[nUpdates];
    worldStartLevels = new int[nWorlds];
  }

  /** pushing a former entry onto the world stack */
  void notifyUpdate(StoredVector vect, int index, Object formerValue) {
    vectorStack[currentLevel] = vect;
    indexStack[currentLevel] = index;
    valueStack[currentLevel] = formerValue;
    currentLevel++;
  if (currentLevel == maxUpdates)
    resizeUpdateCapacity();
}

 private void resizeUpdateCapacity() {
   int newCapacity =  ((maxUpdates * 3) / 2);
   // first, copy the stack of variables
   StoredVector[] tmp1 = new StoredVector[newCapacity];
   System.arraycopy(vectorStack, 0, tmp1, 0, vectorStack.length);
   vectorStack = tmp1;
   // then, copy the stack of former values
   Object[] tmp2 = new Object[newCapacity];
   System.arraycopy(valueStack, 0, tmp2, 0, valueStack.length);
   valueStack = tmp2;
   // then, copy the stack of indices
   int[] tmp4 = new int[newCapacity];
   System.arraycopy(indexStack, 0, tmp4, 0, indexStack.length);
   indexStack = tmp4;

   // last update the capacity
   maxUpdates = newCapacity;
 }

  public void resizeWorldCapacity(int newWorldCapacity) {
    int[] tmp = new int[newWorldCapacity];
    System.arraycopy(worldStartLevels, 0, tmp, 0, worldStartLevels.length);
    worldStartLevels = tmp;
    maxWorlds = newWorldCapacity;
 }

  /** moving up to the next world */
  public void worldPush() {
    worldStartLevels[environment.currentWorld + 1] = currentLevel;
  }

  /** moving down to the previous world */
  public void worldPop() {
    while (currentLevel > worldStartLevels[environment.currentWorld]) {
      currentLevel--;
      StoredVector v = vectorStack[currentLevel];
      v._set(indexStack[currentLevel], valueStack[currentLevel]);
    }
  }

  /** comitting a world: merging it with the previous one */
  public void worldCommit() {
    // TODO
    ;
  }

  public int getSize() {
    return 0;
  }
}

