//     ___  ___         PaLM library for Choco system
//    /__ \/ __\        (c) 2001 - 2004 -- Narendra Jussien
//   ____\ |/_____
//  /_____\/_____ \     Explanation based constraint programming tool
// |/   (_)(_)   \|
//       \ /            Version 0.1
//       \ /            January 2004
//       \ /
// ______\_/_______     Contibutors: Franois Laburthe, Hadrien Cambazard, Guillaume Rochart...

package choco.palm.explain;

import choco.palm.integer.PalmIntVar;
import choco.palm.integer.explain.*;
import choco.palm.real.PalmRealVarImpl;
import choco.palm.real.PalmRealVar;
import choco.palm.real.explain.RealBoundExplanation;

import java.util.Set;
import java.util.BitSet;

import choco.Constraint;
import choco.ConstraintCollection;

/**
 * Explanation interface.
 */

public interface Explanation extends ConstraintCollection {


  /**
   * Makes an IncInfExplanation from the current explain by adding dependencies.
   * @param inf The previous value of the bound.
   * @param var The involved variable.
   */

  public IBoundExplanation makeIncInfExplanation(int inf, PalmIntVar var);


  /**
   * Makes a DecSupExplanation from the current explain by adding dependencies.
   * @param sup The previous value of the bound.
   * @param var The involved variable.
   */

  public IBoundExplanation makeDecSupExplanation(int sup, PalmIntVar var);


  /**
   * Makes a RemovalExplanation from the current explain by adding dependencies.
   * @param value The removed value of the domain.
   * @param var The involved variable.
   */

  public IRemovalExplanation makeRemovalExplanation(int value, PalmIntVar var);


  /**
   * Makes an IncInfExplanation from the current explain by adding dependencies.
   * @param inf The previous value of the bound.
   * @param var The involved variable.
   */

  public RealBoundExplanation makeIncInfExplanation(double inf, PalmRealVar var);


  /**
   * Makes a DecSupExplanation from the current explain by adding dependencies.
   * @param sup The previous value of the bound.
   * @param var The involved variable.
   */

  public RealBoundExplanation makeDecSupExplanation(double sup, PalmRealVar var);


  /**
   * Posts a restoration prop.
   * @param constraint
   */

  public void postUndoRemoval(Constraint constraint);


  /**
   * Creates a set with all the constraints in the explain..
   * @return The explain as a set.
   */

  public Set toSet();


  /**
   * Copies the explain set and returns the new bitset.
   * @return The explain as a BitSet.
   */

  public BitSet getBitSet();


  /**
   * Checks if the explain is valid, that is wether all the constraint are active.
   */

  public boolean isValid();


  /**
   * Checks if the explain is valid, that is wether all the constraint are active. Moreover,
   * it checks that all constraints were poseted before the specified date.
   */

  public boolean isValid(int date);


  /**
   * Removes all constraints from this explanation.
   */

  public void empties();

}
