package choco.palm.integer.constraints;

import choco.palm.integer.AbstractPalmBinIntConstraint;
import choco.palm.integer.PalmIntVar;
import choco.palm.integer.PalmIntDomain;
import choco.palm.explain.PalmConstraintPlugin;
import choco.palm.explain.Explanation;
import choco.palm.PalmProblem;
import choco.integer.IntVar;
import choco.ContradictionException;
import choco.ConstraintCollection;

import java.util.Set;
import java.util.logging.Logger;
import java.util.logging.Level;

/**
 * Created by IntelliJ IDEA.
 * User: Hadrien
 * Date: 18 janv. 2004
 * Time: 19:22:28
 * To change this template use Options | File Templates.
 */
public class PalmElt extends AbstractPalmBinIntConstraint {

  /** uses the cste slot: l[i + cste] = x
   *  (ex: cste = 1 allows to use and index from 0 to length(l) - 1
   */
  protected final int cste;
  protected int[] lvals;

    /**
     * Element constraint
     * (accessing the ith element in a list of values, where i is a variable)
     * the slot v0 represents the index and the slot v1 represents the value
     * propagation with complete arc consistency from values to indices (v1 to v0)
     * propagation with interval approximation from indices to values (v0 to v1)
     * @param v0
     * @param v1
     * @param cste
     * @param lvals
     */
  public PalmElt(IntVar v0, IntVar v1, int cste, int[] lvals) {
        this.v0 = v0;
        this.v1 = v1;
        this.cste = cste;
        this.lvals = lvals;
        this.hook = new PalmConstraintPlugin(this);
  }

  public void updateValueFromIndex() {
    int minVal = Integer.MAX_VALUE, maxVal = Integer.MIN_VALUE, val, j = 0;
    boolean found = false;
    ConstraintCollection e = ((PalmProblem)this.getProblem()).makeExplanation();
    ((PalmConstraintPlugin) this.hook).self_explain(e);
    ((PalmIntVar) this.v0).self_explain(PalmIntDomain.DOM,e);
    int[] values = ((PalmIntVar) this.v0).getAllValues();
    for (int i = 0; i < values.length; i++) {
        val = lvals[cste + values[i]];
        if (minVal > val) minVal = val;
        if (maxVal < val) maxVal = val;
    }
    ((PalmIntVar) v1).updateSup(maxVal, this.cIdx1 ,(Explanation) e.copy());
    ((PalmIntVar) v1).updateInf(minVal, this.cIdx1 ,(Explanation) e.copy());

    values = ((PalmIntVar) this.v0).getAllValues();
    int[] values2 = ((PalmIntVar) this.v1).getAllValues();  // TODO : remplacer par des itrateurs
    // propagate on holes
    if (v1.hasEnumeratedDomain()) {
       for (int i = 0; i < values2.length; i++) {
          while (!found & j < values.length) {
              if (lvals[values[j] + cste] == values2[i])
                 found = true;
              j++;
          }
          if (!found) ((PalmIntVar)v1).removeVal(values2[i],this.cIdx1, (Explanation) e.copy());
          found = false; j = 0;
       }
    }
  }

   public void updateIndexFromValue() {
     ConstraintCollection e = ((PalmProblem)this.getProblem()).makeExplanation();
     ((PalmIntVar) this.v1).self_explain(PalmIntDomain.DOM,e);
     ((PalmConstraintPlugin) this.hook).self_explain(e);
     int minFeasibleIndex = v0.getInf(), maxFeasibleIndex = v0.getSup(), thecause = -1;
     if (v0.getInf() < cste) minFeasibleIndex = cste;
     if (v0.getSup() > (lvals.length - 1)) maxFeasibleIndex = lvals.length - 1;
     if (v1.hasEnumeratedDomain()) thecause = cIdx0;
     while((minFeasibleIndex < (lvals.length - 1)) & v0.canBeInstantiatedTo(minFeasibleIndex) &
           ! v1.canBeInstantiatedTo(lvals[minFeasibleIndex + cste])) minFeasibleIndex++;
     ((PalmIntVar)v0).updateInf(minFeasibleIndex,thecause,(Explanation) e.copy());
     while((maxFeasibleIndex > 0) & v0.canBeInstantiatedTo(maxFeasibleIndex) &
           ! v1.canBeInstantiatedTo(lvals[maxFeasibleIndex + cste])) maxFeasibleIndex--;
     ((PalmIntVar)v0).updateSup(maxFeasibleIndex,thecause,(Explanation) e.copy());
     if (v0.hasEnumeratedDomain()) {
        for (int i = minFeasibleIndex + 1; i < maxFeasibleIndex; i++) {
            if (v0.canBeInstantiatedTo(i) & !v1.canBeInstantiatedTo(lvals[i + cste])) {
              Explanation expl = ((PalmProblem)this.getProblem()).makeExplanation();
              ((PalmConstraintPlugin) this.hook).self_explain(expl);
              ((PalmIntVar) this.v1).self_explain(PalmIntDomain.VAL,lvals[i + cste],expl);
              ((PalmIntVar) this.v0).removeVal(i,thecause,expl);
            }
        }
     }
   }

  public void propagate() throws ContradictionException {
    updateIndexFromValue();
    updateValueFromIndex();
  }

  public void awakeOnInf(int idx) {
    if (idx == 0) updateValueFromIndex();
    else updateIndexFromValue();
  }

  public void awakeOnSup(int idx) {
    if (idx == 0) updateValueFromIndex();
    else updateIndexFromValue();
  }

  public void awakeOnRem(int idx, int val) {
    if (idx == 0) updateValueFromIndex();
    else updateIndexFromValue();
  }

  public void awakeOnRestore(int idx) throws ContradictionException{  
    ConstraintCollection expl = ((PalmProblem)this.getProblem()).makeExplanation();
    ((PalmConstraintPlugin) this.hook).self_explain(expl);
    ((PalmIntVar) v0).updateInf(cste,cIdx0,(Explanation) expl.copy());
    ((PalmIntVar) v0).updateSup(lvals.length - 1 + cste, cIdx0,(Explanation) expl.copy());
     if (idx == 0) updateIndexFromValue();
     else updateValueFromIndex();
  }

  public void awakeOnRestoreInf(int idx) throws ContradictionException {
   awakeOnRestore(idx);
  }

  public void awakeOnRestoreSup(int idx) throws ContradictionException {
   awakeOnRestore(idx);
  }

  public void awakeOnRestoreVal(int idx, int val) throws ContradictionException {
   awakeOnRestore(idx);
  }

  public Boolean isEntailed() {
    if (Logger.getLogger("choco.palm").isLoggable(Level.WARNING))
      Logger.getLogger("choco.palm").warning("Not Yet implemented : NotEqual.isEntailed");
    return null;
  }

  public boolean isSatisfied() {
    return lvals[v0.getValue() + cste] == v1.getValue();
  }

  public Set whyIsTrue() {
    if (Logger.getLogger("choco.palm").isLoggable(Level.WARNING))
      Logger.getLogger("choco.palm").warning("Not Yet implemented : NotEqual.whyIsTrue");
    return null;
  }

  public Set whyIsFalse() {
    if (Logger.getLogger("choco.palm").isLoggable(Level.WARNING))
      Logger.getLogger("choco.palm").warning("Not Yet implemented : NotEqual.whyIsFalse");
    return null;
  }
}
