//     ___  ___         PaLM library for Choco system
//    /__ \/ __\        (c) 2001 - 2004 -- Narendra Jussien
//   ____\ |/_____
//  /_____\/_____ \     Explanation based constraint programming tool
// |/   (_)(_)   \|
//       \ /            Version 0.1
//       \ /            January 2004
//       \ /
// ______\_/_______     Contibutors: Franois Laburthe, Hadrien Cambazard, Guillaume Rochart...

package choco.palm.integer.constraints;

import choco.integer.constraints.IntLinComb;
import choco.integer.IntVar;
import choco.palm.integer.PalmIntVarListener;
import choco.palm.integer.PalmIntVar;
import choco.palm.integer.PalmIntDomain;
import choco.palm.PalmConstraint;
import choco.palm.PalmProblem;
import choco.palm.prop.PalmEngine;
import choco.palm.explain.Explanation;
import choco.palm.explain.PalmConstraintPlugin;
import choco.ContradictionException;
import choco.util.IntIterator;
import choco.util.Arithm;

import java.util.Set;

/**
 * Created by IntelliJ IDEA.
 * User: rochart
 * Date: Jan 13, 2004
 * Time: 7:58:45 AM
 * To change this template use Options | File Templates.
 */
public class PalmIntLinComb extends IntLinComb implements PalmIntVarListener, PalmConstraint {
  public PalmIntLinComb(IntVar[] vars, int[] coeffs, int c, int linOperator) {
    super(vars, coeffs, c, linOperator);
    this.hook = new PalmConstraintPlugin(this);
  }

  public void awakeOnRem(int idx, int x) throws ContradictionException {
    throw new Error("AwakeOnRem in IntLinComb should not be called");
    //this.propagate();
  }

  public void awakeOnVar(int idx) throws ContradictionException {
    throw new Error("Appel au awakeOnvar");
  }

  public void awakeOnRemovals(int idx, IntIterator it) throws ContradictionException {
    this.propagate();
  }

  public void awakeOnRestoreInf(int idx) throws ContradictionException {
    this.propagate();
  }

  public void awakeOnRestoreSup(int idx) throws ContradictionException {
    this.propagate();
  }

  public void awakeOnRestoreVal(int idx, int val) throws ContradictionException {
    throw new Error("AwakeOnRestoreVal in IntLinComb should not be called");
  }

  public void awakeOnRestoreVal(int idx, IntIterator it) throws ContradictionException {
    this.propagate();
  }

  public Set whyIsTrue() {
    return null;
  }

  public Set whyIsFalse() {
    return null;
  }

  public void updateDataStructuresOnConstraint(int idx, int select, int newValue, int oldValue) {
  }

  public void updateDataStructuresOnRestoreConstraint(int idx, int select, int newValue, int oldValue) {
  }

  public void takeIntoAccountStatusChange(int index) {
  }

  private Explanation explainVariablesLB() {
    Explanation expl = ((PalmProblem) this.getProblem()).makeExplanation();
    ((PalmConstraintPlugin) this.hook).self_explain(expl);
    for (int i = 0; i < nbPosVars; i++) {
      ((PalmIntVar) this.vars[i]).self_explain(PalmIntDomain.INF, expl);
    }
    for (int i = nbPosVars; i < this.getNbVars(); i++) {
      ((PalmIntVar) this.vars[i]).self_explain(PalmIntDomain.SUP, expl);
    }
    return expl;
  }

  private Explanation explainVariablesUB() {
    Explanation expl = ((PalmProblem) this.getProblem()).makeExplanation();
    ((PalmConstraintPlugin) this.hook).self_explain(expl);
    for (int i = 0; i < nbPosVars; i++) {
      ((PalmIntVar) this.vars[i]).self_explain(PalmIntDomain.SUP, expl);
    }
    for (int i = nbPosVars; i < this.getNbVars(); i++) {
      ((PalmIntVar) this.vars[i]).self_explain(PalmIntDomain.INF, expl);
    }
    return expl;
  }

  protected boolean propagateNewLowerBound(int mylb) throws ContradictionException {
    Explanation expl = this.explainVariablesLB();
    boolean anyChange = false;
    int nbVars = getNbVars();
    if (mylb > 0) {
      logger.finer("lb = " + mylb + " > 0 => fail");
      ((PalmEngine) this.getProblem().getChocEngine()).raisePalmFakeContradiction(expl);
    }
    int i;
    for (i = 0; i < nbPosVars; i++) {
      int newSupi = Arithm.divFloor(-(mylb), coeffs[i]) + vars[i].getInf();
      if (((PalmIntVar) vars[i]).updateSup(newSupi, cIndices[i], expl)) {
        logger.finer("SUP(" + vars[i].toString() + ") <= " + -(mylb) + "/" + coeffs[i] + " + " + vars[i].getInf() + " = " + newSupi);
        anyChange = true;
      }
    }
    for (i = nbPosVars; i < nbVars; i++) {
      int newInfi = Arithm.divCeil(mylb, -(coeffs[i])) + vars[i].getSup();
      if (((PalmIntVar) vars[i]).updateInf(newInfi, cIndices[i], expl)) {
        logger.finer("INF(" + vars[i].toString() + ") >= " + mylb + "/" + -(coeffs[i]) + " + " + vars[i].getSup() + " = " + newInfi);
        anyChange = true;
      }
    }
    return anyChange;
  }

  protected boolean propagateNewUpperBound(int myub) throws ContradictionException {
    Explanation expl = this.explainVariablesUB();
    boolean anyChange = false;
    int nbVars = getNbVars();
    if (myub < 0) {
      logger.finer("ub = " + myub + " < 0 => fail");
      ((PalmEngine) this.getProblem().getChocEngine()).raisePalmFakeContradiction(expl);
    }
    int i;
    for (i = 0; i < nbPosVars; i++) {
      int newInfi = Arithm.divCeil(-(myub), coeffs[i]) + vars[i].getSup();
      if (((PalmIntVar) vars[i]).updateInf(newInfi, cIndices[i], expl)) {
        logger.finer("INF(" + vars[i].toString() + ") >= " + -(myub) + "/" + coeffs[i] + " + " + vars[i].getSup() + " = " + newInfi);
        anyChange = true;
      }
    }
    for (i = nbPosVars; i < nbVars; i++) {
      int newSupi = Arithm.divFloor(myub, -(coeffs[i])) + vars[i].getInf();
      if (((PalmIntVar) vars[i]).updateSup(newSupi, cIndices[i], expl)) {
        logger.finer("SUP(" + vars[i].toString() + ") <= " + myub + "/" + -(coeffs[i]) + " + " + vars[i].getInf() + " = " + newSupi);
        anyChange = true;
      }
    }
    return anyChange;
  }

}
