//     ___  ___         PaLM library for Choco system
//    /__ \/ __\        (c) 2001 - 2004 -- Narendra Jussien
//   ____\ |/_____
//  /_____\/_____ \     Explanation based constraint programming tool
// |/   (_)(_)   \|
//       \ /            Version 0.1
//       \ /            January 2004
//       \ /
// ______\_/_______     Contibutors: Franois Laburthe, Hadrien Cambazard, Guillaume Rochart...

package choco.palm.real.exp;

import choco.palm.real.PalmRealInterval;
import choco.palm.explain.Explanation;
import choco.palm.PalmProblem;
import choco.real.exp.RealIntervalConstant;
import choco.ConstraintCollection;

/**
 * Implementation of a constant interval.
 */
public class PalmRealIntervalConstant extends RealIntervalConstant implements PalmRealInterval {
  /**
   * The lower bound explanation if needed.
   */
  Explanation explanationOnInf;

  /**
   * The upper bound explanation if needed.
   */
  Explanation explanationOnSup;

  /**
   * Creates a constant from the current state of an interval.
   * @param pb The current problem (needed for creating explanations).
   */
  public PalmRealIntervalConstant(PalmRealInterval interval, PalmProblem pb) {
    super(interval);
    explanationOnInf = pb.makeExplanation();
    explanationOnSup = pb.makeExplanation();
    interval.self_explain(INF, explanationOnInf);
    interval.self_explain(SUP, explanationOnSup);
  }

  /**
   * Creates a constant with the specified values.
   */
  public PalmRealIntervalConstant(double inf, double sup, Explanation expOnInf, Explanation expOnSup) {
    super(inf, sup);
    explanationOnInf = expOnInf;
    explanationOnSup = expOnSup;
  }

  /**
   * Creates a constant without explanations from the current state of the interval.
   */
  public PalmRealIntervalConstant(PalmRealInterval interval) {
    super(interval);
    explanationOnInf = null;
    explanationOnSup = null;
  }

  /**
   * Creates a constant without explanations.
   */
  public PalmRealIntervalConstant(double inf, double sup) {
    super(inf, sup);
    explanationOnInf = null;
    explanationOnSup = null;
  }

  /**
   * Explains the values of the constant.
   * @param select The part if the domain that should be explained.
   * @param e The constraint collection in which this explanation should be added.
   */
  public void self_explain(int select, ConstraintCollection e) {
    if (explanationOnInf != null) switch (select) { // Sinon explication vide...
      case INF: e.merge((ConstraintCollection) this.explanationOnInf); break;
      case SUP: e.merge((ConstraintCollection) this.explanationOnSup); break;
      case DOM:
        e.merge((ConstraintCollection) this.explanationOnInf);
        e.merge((ConstraintCollection) this.explanationOnSup);
        break;
    }
  }
}
