//     ___  ___         PaLM library for Choco system
//    /__ \/ __\        (c) 2001 - 2004 -- Narendra Jussien
//   ____\ |/_____
//  /_____\/_____ \     Explanation based constraint programming tool
// |/   (_)(_)   \|
//       \ /            Version 0.1
//       \ /            January 2004
//       \ /
// ______\_/_______     Contibutors: Franois Laburthe, Hadrien Cambazard, Guillaume Rochart...

package choco.palm.search;

import choco.*;
import choco.branch.AbstractBranching;
import choco.branch.ObjectBranching;
import choco.palm.PalmProblem;

import java.util.List;
import java.util.LinkedList;

/**
 * Abstract class for branching algorithms.
 */

public abstract class PalmAbstractBranching extends AbstractBranching implements ObjectBranching {

  /**
   * The extender the branching which uses this branching.
   */

  protected PalmExtend extender;


  /**
   * Gets the extender which uses this branching.
   */

  public PalmExtend getExtender() {
    return extender;
  }


  /**
   * Sets the extender which uses this branching.
   */

  public void setExtender(PalmExtend extender) {
    this.extender = extender;
  }


  /**
   * Checks if decisions are acceptable.
   * @param csts The decisions that will be taken.
   */

  public abstract boolean checkAcceptable(List csts);


  /**
   * Learns from rejection for unacceptable decisions.
   */

  public abstract void learnFromRejection();

  /*
   * Computes decisions authorized by the learner that can be taken on the specified item by the solver.
   * @param var The item the solver branchs on.
   */
  public List selectAuthorizedDecisions(Object item) {
      PalmLearn learner = extender.manager.learning;
      List decisionlist = (List) getNextBranch(item, null);
      while (!checkAcceptable(decisionlist) | !learner.checkAcceptable(decisionlist)) {
          learnFromRejection();
          decisionlist = (List) getNextBranch(item, decisionlist);
      }
      return decisionlist;
  }


}
