package choco.palm.search.pathrepair;

import choco.palm.search.PalmLearn;
import choco.palm.search.PalmGlobalSearchSolver;
import choco.palm.explain.Explanation;
import choco.palm.explain.PalmConstraintPlugin;
import choco.palm.PalmProblem;
import choco.palm.integer.constraints.PalmAssignment;
import choco.palm.prop.PalmEngine;
import choco.ContradictionException;
import choco.Constraint;
import choco.ConstraintCollection;

import java.util.*;

public class PathRepairLearn extends PalmLearn {

  protected int maxMoves = 1500;
  protected LinkedList explanations;
  protected int maxSize = 7;

  public PathRepairLearn(int lSize, int nMoves) {
    this.maxMoves = nMoves;
    this.maxSize = lSize;
    this.explanations = new LinkedList();
  }

  /**
   *  Update the tabou list of nogood
   * @param nogood
   */
  public void addForbiddenSituation(ConstraintCollection nogood) {
      if (explanations.size() == maxSize) {
          explanations.removeLast();   // Il n'y a pas besoin de faire un reset des serachInfo ici ?
      }
      explanations.addFirst(nogood);
  }

  /**
   * Check if the tabou list contains the given nogood
   * @param nogood
   */
  public boolean isForbidden(ConstraintCollection nogood) {
      Iterator it = explanations.listIterator();
      for (; it.hasNext();) {
         if (((Explanation) nogood).containsAll((Explanation)it.next()))
             return true;
      }
      return false;
  }


  public void learnFromContradiction(Explanation expl) throws ContradictionException  {
      PalmGlobalSearchSolver man =  this.getManager();
      PalmProblem pb =  ((PalmProblem)man.getProblem());
      //TODO : Check limit
      //if (man.getRuntimeStatistics(PalmProblem.RLX) > maxMoves) {
      // ((PalmEngine) pb.getPropagationEngine()).raiseSystemContradiction();
      //}
      BitSet bset = expl.getBitSet();    // iterate on the bitset and avoid HashSet !!!!!!
      Explanation nogood = pb.makeExplanation();  // create the nogood associated to the conflict
      for (int i = bset.nextSetBit(0); i >= 0; i = bset.nextSetBit(i + 1)) {
        Constraint ct = pb.getConstraintNb(i);
          if (((PalmConstraintPlugin)(ct).getPlugIn()).getWeight() == 0)
              nogood.add(ct);
      }
      addForbiddenSituation(nogood);                    // add it in the tabou list
      informConstraintsInExplanation(nogood);
      //System.out.print("Nogood obtenu : ");
      //debugNogood(nogood);
      //System.out.println();
  }

  /**
   * maintain the searchInfo parameter on each constraint concerned by the conflict
   * @param expl
   */
  public void informConstraintsInExplanation(Explanation expl) {
       if (!expl.isEmpty()) {
           PalmProblem pb =  ((PalmProblem) this.getManager().getProblem());
           float sCoef = 1 / (float) expl.size();
           //Iterator it = expl.toSet().iterator();
           BitSet bset = expl.getBitSet();    // iterate on the bitset and avoid HashSet !!!!!!
           for (int i = bset.nextSetBit(0); i >= 0; i = bset.nextSetBit(i + 1)) {
            Constraint ct = pb.getConstraintNb(i);
            PathRepairSearchInfo sInfo = (PathRepairSearchInfo) ((PalmConstraintPlugin)(ct).getPlugIn()).getSearchInfo();
                if (sInfo == null) { // TODO : faire ca autrement
                   sInfo = new PathRepairSearchInfo();
                   ((PalmConstraintPlugin)(ct).getPlugIn()).setSearchInfo(sInfo);
                }
                sInfo.add(sCoef);
           }
       }
  }

  public void learnFromRemoval(Constraint ct) {
      PathRepairSearchInfo sInfo = (PathRepairSearchInfo) ((PalmConstraintPlugin)(ct).getPlugIn()).getSearchInfo();
      //assert(sInfo != null);
      sInfo.set(0);
  }

  /**
   * Check if the set of decision is acceptable by the solver
   * Note : should be called : checkAcceptableExtension
   * @param constraints
   */
  public boolean checkAcceptable(List constraints) {
      PalmGlobalSearchSolver man =  this.getManager();
      PalmProblem pb =  ((PalmProblem)man.getProblem());
      ConstraintCollection situation = pb.makeExplanation();
      Iterator it = constraints.iterator();
      for (; it.hasNext();) {
        Constraint ct = (Constraint)it.next();
        if (((PalmConstraintPlugin) ct.getPlugIn()).isEverConnected())
            situation.add(ct);
      }

      situation.merge(man.getState().getPath());
      return !isForbidden(situation);
  }


  public boolean checkAcceptableRelaxation(Constraint ctout) {
      PalmGlobalSearchSolver man =  this.getManager();
      PalmProblem pb =  ((PalmProblem)man.getProblem());
      Constraint ctin = ((PalmAssignment) ctout).negate();
      ConstraintCollection situation = pb.makeExplanation();

      situation.merge(man.getState().getPath());
      situation.delete(ctout);
      //Iterator it = man.getState().getPath().toSet().iterator();
      //for (; it.hasNext();) {
      //  Constraint ct = (Constraint)it.next();
      //  if (ct != ctout) situation.add(ct);
      //}
      if (((PalmConstraintPlugin) ctin.getPlugIn()).isEverConnected())
        situation.add(ctin);
      return !isForbidden(situation);
  }

  private void debugNogood(Explanation nogood) {
    Set no = nogood.toSet();
    Iterator it = no.iterator();
    while(it.hasNext()) {
        Constraint ct = (Constraint) it.next();
        System.out.print(""+ ct + " : " + ((PathRepairSearchInfo)((PalmConstraintPlugin)(ct).getPlugIn()).getSearchInfo()).getWeigth() + " ");
        }
  }

  private void debugMemory() {
      System.out.println("-----------");
      System.out.print("Chemin de decision :");
      debugDecisionPath();
      System.out.println("Memoire de DR :");
      Iterator it = explanations.listIterator();
      for (; it.hasNext();) {
        debugNogood((Explanation)it.next());
        System.out.println();
      }
      System.out.println("-----------");
  }

  private void debugDecisionPath() {
      Iterator it = this.getManager().getState().getPath().toSet().iterator();
      for (; it.hasNext();) {
        Constraint ct = (Constraint)it.next();
        System.out.print(" - "+ct);
      }
      System.out.println();
  }

    public void assertValidSearchInfo(Explanation expl) {
    Iterator it = expl.toSet().iterator();
    for (; it.hasNext();) {
        Constraint ct = (Constraint)it.next();
        if (((PalmConstraintPlugin)(ct).getPlugIn()).getWeight() == 0) {
            PathRepairSearchInfo sInfo = (PathRepairSearchInfo) ((PalmConstraintPlugin)(ct).getPlugIn()).getSearchInfo();
            assert(sInfo != null);
        }
    }
  }
}
