//     ___  ___         PaLM library for Choco system
//    /__ \/ __\        (c) 2001 - 2004 -- Narendra Jussien
//   ____\ |/_____
//  /_____\/_____ \     Explanation based constraint programming tool
// |/   (_)(_)   \|
//       \ /            Version 0.1
//       \ /            January 2004
//       \ /
// ______\_/_______     Contibutors: Franois Laburthe, Hadrien Cambazard, Guillaume Rochart...

package choco.palm.test;

import junit.framework.TestCase;
import choco.palm.PalmProblem;
import choco.palm.PalmSolver;
import choco.integer.IntVar;
import choco.Problem;

public class MagicSquare extends TestCase {
  public void testSquare() {
    //square(3, false, false);
    //square(3, false, true);
    square(5, true, false);
    //square(5, true, true);
  }

   public void testSquare2() {
    square(4,true,true);
   }

  public void square(int n, boolean help, boolean pathRepair) {
    int magic = n * (n * n + 1) / 2;

    Problem myPb = new PalmProblem();

    IntVar[] vars = new IntVar[n * n];
    for (int i = 0; i < n; i++)
      for (int j = 0; j < n; j++) {
        vars[i * n + j] = myPb.makeEnumIntVar("C" + i + "_" + j, 1, n * n);
      }
    IntVar sum = myPb.makeEnumIntVar("S", 1, n * n * (n * n + 1) / 2);

    try {
      if (help) myPb.post(myPb.eq(sum, magic));

      for (int i = 0; i < n * n; i++)
        for (int j = 0; j < i; j++) {
          myPb.post(myPb.neq(vars[i], vars[j]));
        }

      int[] coeffs = new int[n];
      for (int i = 0; i < n; i++) {
        coeffs[i] = 1;
      }

      for (int i = 0; i < n; i++) {
        IntVar[] col = new IntVar[n];
        IntVar[] row = new IntVar[n];

        for (int j = 0; j < n; j++) {
          col[j] = vars[i * n + j];
          row[j] = vars[j * n + i];
        }

        myPb.post(myPb.eq(myPb.scalar(coeffs, row), sum));
        myPb.post(myPb.eq(myPb.scalar(coeffs, col), sum));
      }

      if (pathRepair) {
        ((PalmSolver) myPb.getSolver()).setPathRepairValues(10, 10000);
      }
      myPb.solve();

      System.out.println("Solution:\n" + ((PalmSolver) myPb.getSolver()).getSolution(0));

      int[] col = new int[n];
      int[] lig = new int[n];
      for (int i = 0; i < n; i++) {
        for (int j = 0; j < n; j++) {
          col[i] += vars[j + i * n].getValue();
          lig[i] += vars[j * n + i].getValue();
        }
      }
      for (int i = 0; i < n - 1; i++) {
        assertEquals(col[i], col[i + 1]);   // toutes les lignes sont gales
        assertEquals(lig[i], lig[i + 1]);  // les colonnes
      }
      assertEquals(col[1], lig[1]);           // il suffit d'une ligne gale  une colonne
      for (int i = 0; i < n; i++) {
        for (int j = 0; j < n; j++) {
          if (i != j) assertTrue(vars[i].getValue() != vars[j].getValue());
        }
      }

      System.out.println((pathRepair?"Decision Repair: ":"MAC-DBT: ")
          + "MAGICSQUARE(" + n + ") Solutions: " + myPb.isFeasible());
      ((PalmProblem)myPb).printRuntimeSatistics();
    } catch (Exception e) {
      e.printStackTrace();
      assertTrue(false);
    }
  }
}
