//     ___  ___         PaLM library for Choco system
//    /__ \/ __\        (c) 2001 - 2004 -- Narendra Jussien
//   ____\ |/_____
//  /_____\/_____ \     Explanation based constraint programming tool
// |/   (_)(_)   \|
//       \ /            Version 0.1
//       \ /            January 2004
//       \ /
// ______\_/_______     Contibutors: Franois Laburthe, Hadrien Cambazard, Guillaume Rochart...

package choco.palm.test;

import choco.palm.PalmProblem;
import choco.palm.PalmSolver;
import choco.palm.search.PalmSolution;
import choco.integer.IntVar;
import choco.Problem;

import junit.framework.TestCase;

public class NQueens extends TestCase {

  public void testNQueensAll() {
    // A priori fonctionne jusqu'a au moins 13
    // - 13 : plus de 4000 sec (plus d'une heure)
    // - 12 : 1013 sec
    // - 11 : 180 sec
    // - 10 : 38 sec
    // - 9 : 5 sec

    final int NBQueen = 8;
    final int[] NBSols = new int[]{0, 0, 0, 2, 10, 4, 40, 92, 352, 724, 2680, 14200, 73712};

    Problem myPb = new PalmProblem();

    IntVar[] vars = new IntVar[NBQueen];
    for (int i = 0; i < vars.length; i++) {
      vars[i] = myPb.makeEnumIntVar("C" + (i + 1), 1, NBQueen);
    }

    for (int i = 0; i < NBQueen; i++)
      for (int j = i + 1; j < NBQueen; j++) {
        int k = j - i;
        myPb.post(myPb.neq(vars[i], vars[j]));
        myPb.post(myPb.neq(vars[i], myPb.plus(vars[j], k)));
        myPb.post(myPb.neq(vars[j], myPb.plus(vars[i], k)));
      }

    myPb.solveAll();

    int nbSolutions = myPb.getSolver().getNbSolutions();
    int falseSoluce = 0;

    for (int i = 0; i < nbSolutions; i++) {
      PalmSolution solution = (PalmSolution) ((PalmSolver) myPb.getSolver()).getSolution(i);
      for (int x = 0; x < NBQueen; x++)
        for (int y = x + 1; y < NBQueen; y++) {
          int k = y - x;
          if ((solution.getValue(x) == solution.getValue(y))
              || (solution.getValue(x) == solution.getValue(y) + k)
              || (solution.getValue(y) == solution.getValue(x) + k)) {
            System.out.println("Fausse solution : ");
            System.out.println(solution);
            falseSoluce++;
          }
        }
    }

    assertEquals(falseSoluce, 0);
    assertEquals(NBSols[NBQueen - 1], nbSolutions);
    System.out.println("NBQueen(" + NBQueen + ") Solutions : " + nbSolutions);
    ((PalmProblem)myPb).printRuntimeSatistics();
  }

  public void testNQueensOneMACDBT() {
    solveNQueensOne(false);
  }

  public void testNQueensOneDR() {
    solveNQueensOne(true);
  }

  public void solveNQueensOne(boolean pathRepair) {
    final int NBQueen = 10;

    Problem myPb = new PalmProblem();

    IntVar[] vars = new IntVar[NBQueen];
    for (int i = 0; i < vars.length; i++) {
      vars[i] = myPb.makeEnumIntVar("C" + (i + 1), 1, NBQueen);
    }

    for (int i = 0; i < NBQueen; i++)
      for (int j = i + 1; j < NBQueen; j++) {
        int k = j - i;
        myPb.post(myPb.neq(vars[i], vars[j]));
        myPb.post(myPb.neq(vars[i], myPb.plus(vars[j], k)));
        myPb.post(myPb.neq(vars[j], myPb.plus(vars[i], k)));
      }

    if (pathRepair) {
      ((PalmSolver) myPb.getSolver()).setPathRepairValues(7, 1500);
    }
    myPb.solve();

    if (myPb.isFeasible() == Boolean.TRUE) {
      for (int x = 0; x < NBQueen; x++)
        for (int y = x + 1; y < NBQueen; y++) {
          int k = y - x;
          assertTrue(vars[x].getValue() != vars[y].getValue());
          assertTrue(vars[x].getValue() != vars[y].getValue() + k);
          assertTrue(vars[y].getValue() != vars[x].getValue() + k);
        }
    }

    System.out.println((pathRepair?"DecisionRepair: ":"MAC-DBT: ")
        + "NQueens (" + NBQueen + "): " + myPb.isFeasible());
    ((PalmProblem)myPb).printRuntimeSatistics();
  }

}
