package choco.real.constraint;

import choco.real.RealVar;
import choco.real.exp.RealIntervalConstant;
import choco.integer.IntVar;
import choco.*;
import choco.util.IntIterator;
import choco.bool.AbstractCompositeConstraint;

public class MixedEqXY extends AbstractConstraint implements MixedConstraint {
  protected RealVar realVar;
  protected int realIdx;
  protected IntVar intVar;
  protected int intIdx;

  public MixedEqXY(RealVar v0, IntVar v1) {
    this.problem = v0.getProblem();
    realVar = v0;
    intVar = v1;
  }

  public RealVar getRealVar(int i) {
    if (i == 0) return realVar;
    return null;
  }

  public int getRealVarNb() {
    return 1;
  }

  public IntVar getIntVar(int i) {
    if (i == 0) return intVar;
    return null;
  }

  public int getIntVarNb() {
    return 1;
  }

  public int getNbVars() {
    return 2;
  }

  public Var getVar(int i) {
    if (i == 0) return realVar;
    if (i == 1) return intVar;
    return null;
  }

  public void setConstraintIndex(int i, int idx) {
    if (i == 0) realIdx = idx;
    else if (i == 1) intIdx = idx;
    else throw new Error("bug in setConstraintIndex i:" + i + " this: " + this);
  }

  public int getConstraintIdx(int idx) {
    if (idx == 0) return realIdx;
    if (idx == 1) return intIdx;
    return -1;
  }

  public int assignIndices(AbstractCompositeConstraint root, int i) {
    int j=i;
    for (int k=0; k<getNbVars(); k++) {
      j++;
      connectVar((AbstractVar) getVar(k), j);
      setConstraintIndex(k, getVar(k).getNbConstraints() - 1);
    }
    return j;
  }

  public boolean isCompletelyInstantiated() {
    return intVar.isInstantiated() && realVar.isInstantiated();
  }

  public boolean isConsistent() {
    return intVar.getInf() <= realVar.getSup() && realVar.getInf() <= intVar.getSup();
  }

  public boolean isSatisfied() {
    return isConsistent();
  }

  public void propagate() throws ContradictionException {
    if (realVar.getInf() > intVar.getInf()) {
      updateIInf();
    }
    if (realVar.getSup() < intVar.getSup()) {
      updateISup();
    }
    updateReal();
  }

  public void awakeOnInf(int idx) throws ContradictionException {
    if (idx == 0) {
      if (realVar.getInf() > intVar.getInf()) {
        updateIInf();
        updateReal();
      }
    } else if (idx == 1) {
      updateReal();
    }
  }

  public void awakeOnSup(int idx) throws ContradictionException {
    if (idx == 0) {
      if (realVar.getSup() < intVar.getSup()) {
        updateISup();
        updateReal();
      }
    } else if (idx == 1) {
      updateReal();
    }
  }

  public void awakeOnInst(int varIdx) throws ContradictionException {
    if (varIdx == 1) {
      updateReal();
    }
  }

  public void awakeOnRem(int varIdx, int val) throws ContradictionException {
  }

  public void awakeOnRemovals(int varIdx, IntIterator deltaDomain) throws ContradictionException {
  }

  public void awakeOnBounds(int varIdx) throws ContradictionException {
    if (varIdx == 0) {
      if (realVar.getInf() > intVar.getInf()) {
        updateIInf();
      }
      if (realVar.getSup() < intVar.getSup()) {
        updateISup();
      }
      updateReal();
    } else if (varIdx == 1) {
      updateReal();
    }
  }

  protected void updateIInf() throws ContradictionException {
    intVar.updateInf((int) Math.ceil(realVar.getInf()), intIdx);
  }

  protected void updateISup() throws ContradictionException {
    intVar.updateSup((int) Math.floor(realVar.getSup()), intIdx);
  }

  protected void updateReal() throws ContradictionException {
    realVar.intersect(new RealIntervalConstant(intVar.getInf(), intVar.getSup()), realIdx);
  }
}
