package choco.search;

import choco.ContradictionException;
import choco.Problem;
import choco.branch.IntBranching;
import choco.branch.AbstractBranching;
import choco.branch.AbstractBranching;
import choco.integer.var.IntDomainVar;

import java.util.logging.Logger;
import java.util.logging.Level;

// **************************************************
// *                   J-CHOCO                      *
// *   Copyright (C) F. Laburthe, 1999-2003         *
// **************************************************
// *  an open-source Constraint Programming Kernel  *
// *     for Research and Education                 *
// **************************************************

/* File choco.search.AbstractIntBranching.java, last modified by flaburthe 20 oct. 2003 12:00:27 */

/**
 * An abstract class for all implementations of branching objets (objects controlling the tree search)
 */
public abstract class AbstractIntBranching extends AbstractBranching implements IntBranching {

  /**
   * a method performing the exploration at the choice point level
   */
  public boolean explore(int n) {
    Problem pb = manager.problem;
    Object v = selectBranchingObject();
    if (v != null) {
      try {
        return branchOn(v, n);
      } catch (ContradictionException e) {
        return false;
      }
    } else if (nextBranching != null) {
      return ((IntBranching) nextBranching).explore(n);
    } else {
      manager.recordSolution();
      manager.showSolution();
      return manager.stopAtFirstSol;
    }
  }

  public void goDownBranch(Object x, int i) throws ContradictionException {
    if (logger.isLoggable(Level.FINE)) {
      int n = manager.problem.getEnvironment().getWorldIndex();
      if (n <= manager.loggingMaxDepth) {
        logger.fine("!! world " + n + ": on " + x + " down branch " + i);
      }
    }
  }

  public void goUpBranch(Object x, int i) throws ContradictionException {
    if (logger.isLoggable(Level.FINE)) {
      int n = manager.problem.getEnvironment().getWorldIndex();
      if (n <= manager.loggingMaxDepth) {
        logger.fine("!! world " + n + ": on " + x + " up branch " + i);
      }
    }
  }
}
