package choco.search;

import choco.ContradictionException;
import choco.integer.var.IntDomainVar;
import choco.integer.search.AbstractIntVarSelector;
import choco.integer.search.IValIterator;

import java.util.logging.Level;

// **************************************************
// *                   J-CHOCO                      *
// *   Copyright (C) F. Laburthe, 1999-2003         *
// **************************************************
// *  an open-source Constraint Programming Kernel  *
// *     for Research and Education                 *
// **************************************************

/* File choco.search.AssignVar.java, last modified by flaburthe 21 oct. 2003 11:58:59 */

public final class AssignVar extends AbstractLargeIntBranching {
  private IVarSelector varHeuristic;
  private IValIterator valHeuristic;

  public AssignVar(IVarSelector varSel, IValIterator valHeuri) {
    varHeuristic = varSel;
    valHeuristic = valHeuri;
  }

  /**
   * selecting the object under scrutiny (that object on which an alternative will be set)
   * @return the object on which an alternative will be set (often  a variable)
   */
  public Object selectBranchingObject() {
    return varHeuristic.selectVar();
  }

  public boolean finishedBranching(Object x, int i) {
    return (!valHeuristic.hasNextVal((choco.integer.var.IntDomainVar) x, i));
  }

  public int getFirstBranch(Object x) {
    return valHeuristic.getFirstVal((choco.integer.var.IntDomainVar) x);
  }

  public int getNextBranch(Object x, int i) {
    return valHeuristic.getNextVal((choco.integer.var.IntDomainVar) x, i);
  }

  public void goDownBranch(Object x, int i) throws ContradictionException {
    super.goDownBranch(x,i);
    choco.integer.var.IntDomainVar y = (choco.integer.var.IntDomainVar) x;
    y.setVal(i);
    manager.problem.propagate();
  }

  public void goUpBranch(Object x, int i) throws ContradictionException {
    super.goUpBranch(x,i);
    choco.integer.var.IntDomainVar y = (choco.integer.var.IntDomainVar) x;
    y.remVal(i);
    manager.problem.propagate();
  }

}
