package choco.set.constraint;

import choco.Var;
import choco.ContradictionException;
import choco.AbstractVar;
import choco.bool.AbstractCompositeConstraint;

// **************************************************
// *                   J-CHOCO                      *
// *   Copyright (C) F. Laburthe, 1999-2003         *
// **************************************************
// *  an open-source Constraint Programming Kernel  *
// *     for Research and Education                 *
// **************************************************

public abstract class AbstractUnSetConstraint extends AbstractSetConstraint{

  /**
   * The unique variable of the constraint.
   */

  public choco.set.SetVar v0;


  /**
   * The index of the constraint among all listeners of its first (and unique)
   * variable.
   */

  public int cIdx0;


  /**
   * Let v be the i-th var of c, records that c is the n-th constraint involving v.
   */

  public void setConstraintIndex(int i, int val) {
    if (i == 0)
      cIdx0 = val;
    else
      throw new Error("bug in setConstraintIndex i:" + i + " this: " + this);
  }


  /**
   * Returns the index of this listeners in the variable <code>idx</code>.
   * @param idx Index of the variable.
   */

  public int getConstraintIdx(int idx) {
    if (idx == 0)
      return cIdx0;
    else
      return -1;
  }


  /**
   * Checks if all the variables of the constraint are instantiated.
   */

  public boolean isCompletelyInstantiated() {
    return v0.isInstantiated();
  }


  /**
   * Returns the number of variables: 1 for an unIntConstraint.
   */

  public int getNbVars() {
    return (1);
  }


  /**
   * Returns the variable number <code>i</code>. Here, <code>i</code>
   * should be 0.
   */

  public Var getVar(int i)  {
    if (i == 0)
      return v0;
    else
      return null;
  }


  /**
   * Adds this constraint as new constraint in each variable.
   */

  public void addConstraint() {
    setConstraintIndex(0, v0.addConstraint(this, 0, true));
    if (this.hook != null) this.hook.addListener();
  }


  /**
   * Deactivates this listener in all the variables.
   */

  public void deactivateConstraint() {
    v0.deactivateConstraint(cIdx0);
    if (this.hook != null) this.hook.deactivateListener();
  }


  /**
   * Activates this listener in all the variables.
   */

  public void activateConstraint() {
    v0.activateConstraint(cIdx0);
    if (this.hook != null) this.hook.activateListener();
  }


  /**
   * Checks if the constraint is active.
   */

  public boolean isActive() {
    return v0.isActive(cIdx0);
  }


  /**
   * Gets the <code>i</code>th search valued variable.
   */

  public choco.set.SetVar getSetVar(int i) {
    if (i == 0)
      return v0;
    else
      return null;
  }

  public int assignIndices(AbstractCompositeConstraint root, int i) {
    int j=i;
    j++; root.connectVar((AbstractVar) v0, j);
    setConstraintIndex(0, v0.getNbConstraints() - 1);
    return j;
  }
}
