package choco.test.real;

import choco.Problem;
import choco.Solver;
import choco.Solution;
import choco.real.RealVar;
import choco.real.RealExp;
import choco.real.search.AssignInterval;
import choco.real.search.CyclicRealVarSelector;
import choco.real.constraint.Equation;
import choco.real.exp.RealMinus;
import choco.real.exp.RealCos;
import choco.real.exp.RealSin;
import choco.real.exp.RealIntervalConstant;
import junit.framework.TestCase;

public class TrigoTests extends TestCase  {
  public void test1() {
    Problem pb = new Problem();
    RealVar alpha = pb.makeRealVar("alpha", - Math.PI, Math.PI);

    RealExp exp = new RealMinus(pb,
        new RealCos(pb, alpha),
        new RealSin(pb, alpha)
        );
    pb.post(new Equation(pb, exp));

    boolean first = false;
    Solver solver = pb.getSolver();
    solver.setFirstSolution(first);
    solver.generateSearchSolver(pb);
    solver.addGoal(new AssignInterval(new CyclicRealVarSelector(pb)));
    solver.launch();

    assertTrue(solver.getNbSolutions() >= 2);
    assertTrue(Math.abs(Math.cos(alpha.getInf()) - Math.sin(alpha.getInf())) < 1e-8);
  }

  public void test2() {
    Problem pb = new Problem();
    RealVar alpha = pb.makeRealVar("alpha", - 5.5 * Math.PI, - 1.5 * Math.PI);

    RealExp exp = new RealCos(pb, alpha);
    pb.post(new Equation(pb, exp, new RealIntervalConstant(1.0, 1.0)));

    boolean first = false;
    Solver solver = pb.getSolver();
    solver.setFirstSolution(first);
    solver.generateSearchSolver(pb);
    solver.addGoal(new AssignInterval(new CyclicRealVarSelector(pb)));
    solver.launch();

    assertTrue(solver.getNbSolutions() >= 2);
  }
}
