// **************************************************
// *                   J-CHOCO                      *
// *   Copyright (C) F. Laburthe, 1999-2003         *
// **************************************************
// *  an open-source Constraint Programming Kernel  *
// *     for Research and Education                 *
// **************************************************

/* File choco.test.search.SolveTest.java, last modified by Francois 2 dc. 2003 23:49:19 */
package choco.test.search;

import choco.Problem;
import choco.integer.IntVar;
import junit.framework.TestCase;

import java.util.logging.Logger;

public class SolveTest extends TestCase {
  private Logger logger = Logger.getLogger("choco.test.search.SolveTest");
  private Problem pb;
  private choco.integer.IntVar x;
  private choco.integer.IntVar y;

  protected void setUp() {
    logger.fine("StoredInt Testing...");
    pb = new Problem();
    x = pb.makeBoundIntVar("X", 0, 5);
    y = pb.makeBoundIntVar("Y", 0, 1);
  }

  protected void tearDown() {
    x = null;
    y = null;
    pb = null;
  }

  /**
   * test the solution count for an infeasible problem
   */
  public void test1() {
    logger.finer("test1");
    pb.post(pb.eq(x, 2));
    pb.post(pb.eq(x, 3));
    pb.solve(false);
    assertEquals(pb.getSolver().getNbSolutions(), 0);
    assertEquals(pb.isFeasible(), Boolean.FALSE);
  }

  /**
   * test the solution count for instantiated problem
   */
  public void test2() {
    logger.finer("test2");
    pb.post(pb.eq(x, 2));
    pb.post(pb.eq(y, 1));
    pb.solve(true);
    assertEquals(pb.isFeasible(), Boolean.TRUE);
    assertEquals(pb.getSolver().getNbSolutions(), 1);
  }

  /**
   * test the solution count for a simple one-variable problem
   */
  public void test3() {
    logger.finer("test3");
    pb.post(pb.eq(y, 1));
    pb.solve(true);
    assertEquals(pb.isFeasible(), Boolean.TRUE);
    assertEquals(pb.getSolver().getNbSolutions(), 6);
  }

  /**
   * test the solution count for a simple two-variable problem
   */
  public void test4() {
    logger.finer("test4");
    pb.solve(true);
    assertEquals(pb.isFeasible(), Boolean.TRUE);
    assertEquals(pb.getSolver().getNbSolutions(), 12);
  }
}

