package choco.test.set;

import choco.Problem;
import choco.Constraint;
import choco.ContradictionException;
import choco.Solution;
import choco.integer.IntVar;
import choco.set.SetVar;
import choco.set.SetModeler;
import choco.set.search.AssignSetVar;
import choco.set.search.MinDomSet;
import choco.set.search.MinEnv;
import choco.set.constraint.*;

import java.util.logging.Logger;
import java.util.logging.Level;

import junit.framework.TestCase;

// **************************************************
// *                   J-CHOCO                      *
// *   Copyright (C) F. Laburthe, 1999-2003         *
// **************************************************
// *  an open-source Constraint Programming Kernel  *
// *     for Research and Education                 *
// **************************************************

public class SearchTests extends TestCase{

   private Logger logger = Logger.getLogger("choco.test.search.EqualXCTest");
  private Problem pb;
  private SetVar x;
  private SetVar y;
  private SetVar z;
  private Constraint c1;
  private Constraint c2;
  private Constraint c3;

  protected void setUp() {
    logger.fine("EqualXC Testing...");
    pb = new Problem();
  }

  protected void tearDown() {
    c1 = null;
    c2 = null;
    c3 = null;
    x = null;
    y = null;
    z = null;
    pb = null;
  }

  /**
   *  A ternary Steiner system of order n is a set of triplets of distinct elements
   *  taking their values between 1 and n, such that all the pairs included in two different triplets are different.
   *  une solution pour n = 7 : [{1, 2, 3}, {2, 4, 5}, {3, 4, 6}, {1, 4, 7}, {1, 5, 6}, {2,6, 7}, {3, 5, 7}]
   *  Il faut que n % 6 = 1 ou n % 6 = 3 pour n soit une valeur valide pour le pb
   */
   public void steinerSystem(int m) {
    pb = new Problem();
    SetModeler mod = new SetModeler(pb);
    Logger.getLogger("choco.search").setLevel(Level.FINEST);
    int n = m * (m - 1) / 6;
    SetVar[] vars = new SetVar[n];
    SetVar[] intersect = new SetVar[n * n];
    for (int i = 0; i < n; i++)
       vars[i] = pb.makeSetVar("set " + i,1,n);
    for (int i = 0; i < n; i++)
       for (int j = i + 1; j < n; j++)
         intersect[i * n + j] = pb.makeSetVar("interSet " + i + " " + j,1,n);

    for (int i = 0; i < n; i++)
      pb.post(mod.eqCard(vars[i],3));
    for (int i = 0; i < n; i++) {
       for (int j = i + 1; j < n; j++) {
         pb.post(mod.setInter(vars[i],vars[j],intersect[i * n + j]));
         pb.post(mod.leqCard(intersect[i * n + j],1));
       }
    }
    
    pb.getSolver().setFirstSolution(true);
    pb.getSolver().generateSearchSolver(pb);
    pb.getSolver().addGoal(new AssignSetVar(new MinDomSet(pb,vars), new MinEnv(pb)));
    pb.getSolver().launch();
    System.out.println("NbSolution " + pb.getSolver().getNbSolutions());
    Solution sol = (Solution) pb.getSolver().getSearchSolver().solutions.get(0);
    sol.restore();
    for (int i = 0; i < n; i++) {
      System.out.println("set[" + i + "]:" + vars[i].pretty());
    }
   }

   public void test2() {
     steinerSystem(7);
   }

  public void test1() {
    pb = new Problem();
    x = pb.makeSetVar("X", 1, 5);
    y = pb.makeSetVar("Y", 1, 2);
    c1 = new MemberX(x, 3);
    c2 = new MemberX(x, 5);
    c3 = new NotMemberX(x, 2);
    logger.finer("test1");
    try {
      pb.post(c1);
      pb.post(c2);
      pb.post(c3);
      pb.propagate();
    } catch (ContradictionException e) {
      assertTrue(false);
    }
    assertTrue(x.isInDomainKernel(3));
    assertTrue(x.isInDomainKernel(5));
    assertTrue(!x.isInDomainKernel(2));
    pb.getSolver().setFirstSolution(false);
    pb.getSolver().generateSearchSolver(pb);
    pb.getSolver().addGoal(new AssignSetVar(new MinDomSet(pb), new MinEnv(pb)));
    pb.getSolver().launch();

    assertEquals(16, pb.getSolver().getNbSolutions());
  }




}

