package chocosamples;

import choco.Problem;
import choco.Solver;
import choco.integer.IntVar;
import choco.real.RealVar;
import choco.real.RealExp;
import choco.real.RealModeler;
import choco.real.exp.RealIntervalConstant;
import choco.real.constraint.MixedEqXY;
import choco.real.search.AssignInterval;
import choco.real.search.CyclicRealVarSelector;

import java.util.logging.Logger;
import java.util.logging.Level;
import java.util.logging.Handler;
import java.util.logging.ConsoleHandler;

/**
 * Samples in Elisa package.
 */
public class Arm {
  public static void main(String[] args) {
    Problem pb = new Problem();
    RealModeler m = new RealModeler(pb);

    RealVar a = m.makeRealVar("a", 2.0, 8.0);
    RealVar b = m.makeRealVar("b", 2.0, 8.0);
    RealVar alpha = m.makeRealVar("alpha", 0.0, Math.PI);
    RealVar beta = m.makeRealVar("beta", 0.0, Math.PI);
    RealVar x = m.makeRealVar("x", 0.0, 10.0);
    RealVar y = m.makeRealVar("y", 0.0, 8.0);

    IntVar i = pb.makeBoundIntVar("i", 1, 4);
    IntVar j = pb.makeBoundIntVar("j", 1, 4);
    RealVar ir = m.makeRealVar("i'", 1.0, 4.0);
    RealVar jr = m.makeRealVar("j'", 1.0, 4.0);
    pb.post(new MixedEqXY(ir, i));
    pb.post(new MixedEqXY(jr, j));

    RealExp exp1 = m.minus(y, m.plus(m.mult(a, m.sin(alpha)), m.mult(b, m.sin(m.minus(alpha, beta)))));
    RealExp exp2 = m.minus(x, m.plus(m.mult(a, m.cos(alpha)), m.mult(b, m.cos(m.minus(alpha, beta)))));
    pb.post(m.eq(exp1, 0.0));
    pb.post(m.eq(exp2, 0.0));

    pb.post(m.leq(m.mult(a, m.cos(alpha)), 10.0));
    pb.post(m.leq(m.mult(a, m.sin(alpha)), 8.0));

    RealExp circle = m.plus(m.power(m.minus(x, m.cst(8.0)), 2), m.power(m.minus(y, m.cst(4.0)), 2));
    pb.post(m.leq(circle, 4.0));

    pb.post(m.eq(alpha, m.arround(Math.PI/6)));
    pb.post(m.eq(m.minus(a, m.mult(ir, new RealIntervalConstant(1.99, 2.01))), 0.0));
    pb.post(m.eq(m.minus(b, m.mult(jr, new RealIntervalConstant(1.99, 2.01))), 0.0));

    /*Logger.getLogger("choco.search").setLevel(Level.ALL);
    Handler h = new ConsoleHandler();
    h.setLevel(Level.ALL);
    Logger.getLogger("choco").addHandler(h);

    Logger.getLogger("choco.search").fine("test !"); */

    boolean first = true;
    Solver solver = pb.getSolver();
    solver.setFirstSolution(first);
    solver.generateSearchSolver(pb);
    solver.addGoal(new AssignInterval(new CyclicRealVarSelector(pb)));
    solver.launch();
  }
}
