package chocosamples;

// **************************************************
// *                   J-CHOCO                      *
// *   Copyright (C) F. Laburthe, 1999-2003         *
// **************************************************
// *  an open-source Constraint Programming Kernel  *
// *     for Research and Education                 *
// **************************************************

/* File PACKAGE_NAME.Send.java, last modified by Francois 17 nov. 2003 16:03:29 */

import choco.Problem;
import choco.integer.IntVar;

import java.util.logging.Logger;

public class Send {
  private static Logger logger = Logger.getLogger("Send");

  public static void main(String[] args) {
    System.out.println(new Send().demo());
  }

  public String demo() {
    StringBuffer ret = new StringBuffer();
    logger.fine("CHOCO Sample: SEND + MORE = MONEY");
    Problem pb = new Problem();
    IntVar S = pb.makeEnumIntVar("S", 1, 9);
    IntVar E = pb.makeEnumIntVar("E", 0, 9);
    IntVar N = pb.makeEnumIntVar("N", 0, 9);
    IntVar D = pb.makeEnumIntVar("D", 0, 9);
    IntVar M = pb.makeEnumIntVar("M", 1, 9);
    IntVar O = pb.makeEnumIntVar("O", 0, 9);
    IntVar R = pb.makeEnumIntVar("R", 0, 9);
    IntVar Y = pb.makeEnumIntVar("Y", 0, 9);
    IntVar[] letters = {S, E, N, D, M, O, R, Y};
    IntVar SEND = pb.makeBoundIntVar("SEND", 0, 1000000);
    IntVar MORE = pb.makeBoundIntVar("MORE", 0, 1000000);
    IntVar MONEY = pb.makeBoundIntVar("MONEY", 0, 1000000);
    for (int i = 1; i <= 7; i++) {
      for (int j = 0; j < i; j++) {
        pb.post(pb.neq(letters[i], letters[j]));
      }
    }
    pb.post(pb.eq(pb.minus(pb.plus(SEND, MORE), MONEY), 0));
    pb.post(pb.eq(SEND, pb.scalar(new int[]{1000, 100, 10, 1}, new IntVar[]{S, E, N, D})));
    pb.post(pb.eq(MORE, pb.scalar(new int[]{1000, 100, 10, 1}, new IntVar[]{M, O, R, E})));
    pb.post(pb.eq(MONEY, pb.scalar(new int[]{10000, 1000, 100, 10, 1}, new IntVar[]{M, O, N, E, Y})));


    pb.solve(false);
    System.out.println("" + pb.getSolver().getNbSolutions());
    assert(pb.getSolver().getNbSolutions() == 1);
    pb.getSolver().getSearchSolver().restoreBestSolution();

    ret.append("'Send More Money' problem consists of affecting digit to each letter " +
        "S, E, N, D, M, O, R, and Y so that the following equation is true: " +
        "SEND + MORE = MONEY\n\n");
    ret.append("One solution is:\n");
    ret.append("  SEND    " + S.getValue() + "" + E.getValue() + "" +
        N.getValue() + "" + D.getValue() +  "\n");
    ret.append("+ MORE  + " + M.getValue() + "" + O.getValue() + "" +
        R.getValue() + "" + E.getValue() + "\n");
    ret.append(" -----   -----\n");
    ret.append(" MONEY   " + M.getValue() + "" + O.getValue() + "" +
        N.getValue() + "" +  E.getValue() + "" + Y.getValue() + "\n");

    return ret.toString();
  }

}
