package chocosamples;

import choco.Problem;
import choco.Solution;
import choco.set.SetModeler;
import choco.set.SetVar;
import choco.set.search.AssignSetVar;
import choco.set.search.MinDomSet;
import choco.set.search.MinEnv;


// **************************************************
// *                   J-CHOCO                      *
// *   Copyright (C) F. Laburthe, 1999-2003         *
// **************************************************
// *  an open-source Constraint Programming Kernel  *
// *     for Research and Education                 *
// **************************************************

public class SteinerSystem {



  public static void main(String[] args) {
    System.out.println(new SteinerSystem().demo());
  }

  /**
   *  A ternary Steiner system of order n is a set of triplets of distinct elements
   *  taking their values between 1 and n, such that all the pairs included in two different triplets are different.
   *  a solution for n = 7 :
   *  [{1, 2, 3}, {2, 4, 5}, {3, 4, 6}, {1, 4, 7}, {1, 5, 6}, {2,6, 7}, {3, 5, 7}]
   *  we must have n % 6 = 1 or n % 6 = 3 to get a valid n for the problem
   */

  public String demo() {
    int m = 7;
    Problem pb = new Problem();
    SetModeler mod = new SetModeler(pb);
    int n = m * (m - 1) / 6;

    SetVar[] vars = new SetVar[n];
    SetVar[] intersect = new SetVar[n * n];

    // Create Variables
    for (int i = 0; i < n; i++)
       vars[i] = pb.makeSetVar("set " + i,1,n);
    for (int i = 0; i < n; i++)
       for (int j = i + 1; j < n; j++)
         intersect[i * n + j] = pb.makeSetVar("interSet " + i + " " + j,1,n);

    // Post constraints
    for (int i = 0; i < n; i++)
      pb.post(mod.eqCard(vars[i],3));
    for (int i = 0; i < n; i++) {
       for (int j = i + 1; j < n; j++) {
         pb.post(mod.setInter(vars[i],vars[j],intersect[i * n + j]));
         pb.post(mod.leqCard(intersect[i * n + j],1));
       }
    }

    // Search for a solution
    pb.getSolver().setFirstSolution(true);
    pb.getSolver().generateSearchSolver(pb);
    pb.getSolver().addGoal(new AssignSetVar(new MinDomSet(pb,vars), new MinEnv(pb)));
    pb.getSolver().launch();

    // Get the soution
    StringBuffer s = new StringBuffer();
    Solution sol = (Solution) pb.getSolver().getSearchSolver().solutions.get(0);
    sol.restore();
    s.append("A ternary Steiner system of order n is a set of triplets of n*(n - 1) / 6 " +
        "distinct elements taking their values between 1 and n," +
        " such that all the pairs included in two different triplets are different. " +
        " see http://mathworld.wolfram.com/SteinerTripleSystem.html \n \n");

    s.append("A solution for n = " + m + "\n" + "\n");
    for (int i = 0; i < n; i++) {
      s.append("set[" + i + "]:" + vars[i].pretty() + "\n");
    }
    return s.toString();
  }

}
